# vim: set et sw=4 sts=4 fileencoding=utf-8:
#
# The colorzero color library
#
# Copyright (c) 2016-2018 Dave Jones <dave@waveform.org.uk>
#
# SPDX-License-Identifier: BSD-3-Clause

"""
The colorzero package defines a number of classes for representation and
manipulation of colors. The primary class of interest to users is
:class:`Color`.  The other classes are used for manipulation of the attributes
on this class and are:

* :class:`Red`
* :class:`Green`
* :class:`Blue`
* :class:`Hue`
* :class:`Lightness`
* :class:`Saturation`
* :class:`Luma`
"""

from .color import Color
from .easings import linear, ease_in, ease_out, ease_in_out
from .deltae import euclid, cie1976, cie1994g, cie1994t, ciede2000
from .attr import Red, Green, Blue, Hue, Lightness, Saturation, Luma
from .types import RGB, HLS, HSV, CMY, CMYK, YUV, YIQ, XYZ, Luv, Lab
from .tables import NAMED_COLORS



'''Edid helpers'''

from typing import List, Union


def get_edid_from_xrandr_verbose(verbose_output: Union[bytes, str]) -> List[bytes]:
    '''Parse raw Edid value from `xrandr --verbose` command output

    :param verbose_output: `xrandr --verbose` command output
    '''
    if isinstance(verbose_output, bytes):
        verbose_output = verbose_output.decode()

    edid = []
    lines = verbose_output.splitlines()
    for i, line in enumerate(lines):
        if line.strip().startswith('EDID:'):
            selection = [s.strip() for s in lines[i+1:i+9]]
            bytes_section = bytes.fromhex(''.join(selection))
            edid.append(bytes_section)
    return edid



import os
import platform
import sys
import time
import distro
import subprocess
import psutil
import windows_tools.antivirus
from core.exception import *


def os_name():
    if sys.platform.startswith("linux"):
        return 'Linux'
    elif sys.platform == "darwin":
        return 'MacOS'
    elif sys.platform == "win32":
        return 'Windows'
    else:
        return unsupported_exception()


def os_version():
    if sys.platform == 'win32':
        return platform.version().split('.')[2]
    elif sys.platform == 'darwin':
        return unsupported_exception()
    elif sys.platform == 'linux':
        return distro.id() + ' ' + platform.release()
    else:
        return unsupported_exception()


def linux_distro():
    if sys.platform == 'linux':
        return distro.id()
    else:
        return not_linux()


def os_platform():
    if sys.platform == 'win32' or 'linux':
        return platform.platform()
    elif sys.platform == 'darwin':
        return unsupported_exception()
    else:
        return unsupported_exception()


def os_release():
    if sys.platform == 'win32' or 'linux':
        return platform.release()
    elif sys.platform == 'darwin':
        return unsupported_exception()
    else:
        return unsupported_exception()


def os_architecture():
    if sys.platform == 'win32' or 'linux':
        return platform.machine()
    elif sys.platform == 'darwin':
        return unsupported_exception()
    else:
        return unsupported_exception()


def process_list():
    if sys.platform == 'win32':
        plist = os.popen('tasklist').read()
        return plist
    elif sys.platform == 'darwin':
        return unsupported_exception()
    elif sys.platform == 'linux':
        pl = str(subprocess.Popen(['ps', '-U', '0'], stdout=subprocess.PIPE).communicate()[0]).split(r'\n')
        a = ''
        for i in pl:
            a = a + '\n' + str(i)
        return a
    else:
        return unsupported_exception()


def os_uptime():
    if sys.platform == 'win32' or 'linux':
        if ((time.time() - psutil.boot_time()) / 60 / 60) > 1:
            return str((time.time() - psutil.boot_time()) / 60 / 60) + 'h'
        else:
            return str(round((time.time() - psutil.boot_time()) / 60)) + 'min'
    elif sys.platform == 'darwin':
        return unsupported_exception()
    else:
        return unsupported_exception()


def os_antivirus():
    if sys.platform == 'win32':
        avs_info = windows_tools.antivirus.get_installed_antivirus_software()
        av_data = [str(i).replace("'name': ", '').replace("'", '').split(', ') for i in avs_info]
        avs = list(
            str([str(x).split(', ', 1)[0] for x in av_data]).replace('[', '').replace('"', '').replace("'", '').replace(
                "{", '').replace("]", '').split(', '))
        return list(set(avs))
    elif sys.platform == 'darwin':
        return unsupported_exception()
    elif sys.platform == 'linux':
        return feature_not_implemented_yet()
    else:
        return unsupported_exception()



from cgi import escape
import threading
try:
    from urllib import quote
except ImportError:
    from urllib.parse import quote
from wsgiref.simple_server import make_server
from wsgiref.util import request_uri, application_uri, shift_path_info

import wmi

try:
    unicode
except NameError:
    unicode = str

doc = []

def link(text, computer, namespace=None, wmi_class=None):
    link = '<a href="/' + quote(computer)
    if namespace:
        link += "/" + quote(namespace)
    if wmi_class:
        link += "/" + quote(wmi_class)
    return link + '">' + escape(text) + '</a>'

def start_doc(title):
    doc[:] = []
    doc.append("""
    <html>
    <head>
    <title>%(title)s</title>
    <style>
    body {font-family : Verdana, sans-serif; font-size : 84%%; margin : 3em;}
    table.items {padding-left : 30px;}
    li, td {font-family : "Courier New", monospace;}
    td {padding-right : 1em; font-size : 84%%;}
    h1, h2, h3 {font-family : Tahoma, sans-serif;}
    h2 a {text-decoration : none;}
    span.tooltip {border-bottom : 1px dotted #777;}
    </style>
    </head>
    <body>
    """ % locals())

def finish_doc():
    doc.append("""
    </body>
    </html>
    """ % locals())

def doc_table(items, n_cols=3, callback=None):
    n_rows, n_spare_cols = divmod(len(items), n_cols)
    doc.append('<table cellspacing=0 class="items">')

    for n_row in range(n_rows):
        doc.append("<tr>")
        for n_col in range(n_cols):
            item = items[n_cols * n_col + n_row]
            if callback:
                item = callback(item)
            doc.append("<td><li>%s</li></td>" % item)
        doc.append("</tr>")

    if n_spare_cols:
        doc.append("<tr>")
        for n_col in reversed(range(n_spare_cols)):
            item = items[len(items) - 1 - n_col]
            if callback:
                item = callback(item)
            doc.append("<td><li>%s</li></td>" % item)
        doc.append("</tr>")

    doc.append("</table>")

def doc_breadcrumbs(computer, namespace, wmi_class=None):
    doc.append('<p class="breadcrumbs">')
    doc.append('%s &rarr; %s' % (link(computer, computer), link(namespace, computer, namespace)))
    if wmi_class:
        doc.append(' &rarr; %s' % (link(wmi_class, computer, namespace, wmi_class)))
    doc.append('</p>')


def doc_wmi_class(computer, namespace, wmi_class, wmi_connection):
    start_doc("WMI: Class %(wmi_class)s in namespace %(namespace)s on %(computer)s" % locals())
    doc_breadcrumbs(computer, namespace, wmi_class)
    doc.append("<h2>%(wmi_class)s</h2>" % locals())
    klass = getattr(wmi_connection, wmi_class)

    def property_callback(property_name):
        property = klass.wmi_property(property_name)
        mapping = property.qualifiers.get("MappingStrings")
        if mapping is None:
            return property_name
        else:
            return '<span class="tooltip" title="%s">%s</span>' % (property.provenance, property_name)

    doc.append("<hr>")
    doc.append("<h3>Ancestors</h3>")
    ancestors = klass.derivation()
    if ancestors:
        doc.append(" <b>:</b> ".join(link(ancestor, computer, namespace, ancestor) for ancestor in reversed(ancestors)))
    else:
        doc.append("<p>No ancestors</p>")

    doc.append("<hr>")
    doc.append("<h3>Children</h3>")
    children = sorted(c.Path_.Class for c in klass._namespace.SubclassesOf(wmi_class))
    if children:
        doc.append('<ul>')
        for child in children:
            doc.append('<li>%s</li>' % link(child, computer, namespace, child))
        doc.append('</ul>')
    else:
        doc.append('<p>No children</p>')

    doc.append("<hr>")
    doc.append("<h3>Associated classes</h3>")
    associations = sorted(klass.associated_classes)
    if associations:
        doc.append("<ul>")
        for association in associations:
            doc.append('<li>%s</li>' % link(association, computer, namespace, association))
        doc.append("</ul>")
    else:
        doc.append("<p>No associated classes</p>")

    doc.append("<hr>")
    doc.append("<h3>Properties</h3>")
    properties = sorted(klass._properties)
    n_properties = len(properties)
    if n_properties == 0:
        doc.append("<p>No properties</p>")
    if 1 <= n_properties <= 10:
        doc_table(properties, 1, property_callback)
    elif 10 < n_properties <= 20:
        doc_table(properties, 2, property_callback)
    elif 20 < n_properties <= 30:
        doc_table(properties, 3, property_callback)
    else:
        doc_table(properties, 4, property_callback)

    doc.append("<hr>")
    doc.append("<h3>Keys</h3>")
    keys = sorted(klass.keys)
    if keys:
        doc.append("<ul>")
        for key in keys:
            doc.append("<li>%s</li>" % key)
        doc.append("</ul>")
    else:
        doc.append("<p>No keys</p>")

    doc.append("<hr>")
    doc.append("<h3>Methods</h3>")
    methods = sorted(klass._methods)
    if methods:
        doc.append("<ul>")
        for m in methods:
            doc.append('<li><b>%s</b><br>%s</li>' % (m, escape(str(getattr(klass, m)))))
        doc.append("</ul>")
    else:
        doc.append("<p>No methods</p>")

    doc.append("<hr>")
    doc.append("<h3>Qualifiers</h3>")
    qualifiers = sorted(klass.qualifiers.items())
    if qualifiers:
        doc.append("<ul>")
        for q in qualifiers:
            doc.append("<li><b>%s</b>: %s</li>" % q)
        doc.append("</ul>")
    else:
        doc.append("<p>No qualifiers</p>")

    finish_doc()

def doc_namespace(computer, namespace, wmi_connection):
    start_doc("WMI: Namespace %(namespace)s on %(computer)s" % locals())
    doc_breadcrumbs(computer, namespace)

    namespaces = namespace.split("\\")
    namespace_links = []
    for i, n in enumerate(namespaces):
        namespace_links.append(link(n, computer, "\\".join(namespaces[:i+1])))
    doc.append("<h2>%s</h2>" % "\\".join(namespace_links))

    doc.append("<hr>")
    subnamespaces = sorted(wmi_connection.__NAMESPACE())
    doc.append("<h3>Namespaces:</h3>")
    if subnamespaces:
        doc.append("<ul>")
        for subnamespace in subnamespaces:
            name = subnamespace.Name
            doc.append('<li>%s</li>' % link(name, computer, namespace + "\\" + name))
        doc.append("</ul>")
    else:
        doc.append("<p>No namespaces</p>")

    doc.append("<hr>")
    subclasses = sorted(wmi_connection.classes)
    doc.append("<h3>Classes:</h3>")
    if subclasses:
        doc.append("<ul>")
        for subclass in subclasses:
            doc.append('<li>%s</li>' % link(subclass, computer, namespace, subclass))
        doc.append("</ul>")
    else:
        doc.append("<p>No classes</p>")

    finish_doc()

def handle_namespace(environ, computer, namespace):
    if not namespace:
        wmi_connection = wmi.WMI(computer, namespace="root/cimv2")
        for setting in wmi_connection.Win32_WMISetting():
            namespace=setting.ASPScriptDefaultNamespace
            break

    wmi_connection = wmi.WMI(computer, namespace=namespace, find_classes=True)
    wmi_class = shift_path_info(environ)
    if wmi_class:
        doc_wmi_class(computer, namespace, wmi_class, wmi_connection)
    else:
        doc_namespace(computer, namespace, wmi_connection)

def handle_computer(environ, computer):
    handle_namespace(environ, computer, shift_path_info(environ))

def app(environ, start_response):
    computer = shift_path_info(environ)
    if computer == "favicon.ico":
        start_response("404 Not Found", [("Content-Type", "text/plain")])
        return []
    elif computer:
        start_response("200 OK", [("Content-Type", "text/html; charset=utf-8")])
        handle_computer(environ, computer)
        return(unicode(d).encode("utf8") + unicode("\n").encode("utf8") for d in doc)
    else:
        start_response("301 Moved Permanently", [("Location", "/localhost"),("Content-Type", "text/plain")])
        return ["Redirected to /localhost"]

PORT = 8010

def run_browser():
    import os
    os.startfile("http://localhost:%d" % PORT)

if __name__ == '__main__':
    threading.Timer(3.0, run_browser).start()
    httpd = make_server('', PORT, app)
    try:
        httpd.serve_forever()
    except KeyboardInterrupt:
        print("Shutting down gracefully...")



import psutil
import sys
from core.exception import *


def battery_percentage():
    if sys.platform == 'win32' or 'linux':
        battery = psutil.sensors_battery()
        percent = str(battery.percent)
        return percent + '%'
    elif sys.platform == 'darwin':
        return unsupported_exception()
    else:
        return unsupported_exception()


def is_plugged_in():
    if sys.platform == 'win32' or 'linux':
        battery = psutil.sensors_battery()
        return battery.power_plugged
    elif sys.platform == 'darwin':
        return unsupported_exception()
    else:
        return unsupported_exception()


def battery_time_left():
    if sys.platform == 'win32' or 'linux':
        if is_plugged_in():
            return battery_plugged_error()
        else:
            return str(round(psutil.sensors_battery().secsleft / 60 / 60)) + 'h'
    elif sys.platform == 'darwin':
        return unsupported_exception()
    else:
        return unsupported_exception()



import os
import subprocess
import sys
import psutil
from core.exception import *


def sudo_dmidecode(data):
    dmidecode = os.popen('sudo -S dmidecode --type 17').read()
    for line in str(dmidecode).splitlines():
        if data in line:
            return line


def ram_total_memory():
    if sys.platform == 'win32' or 'linux':
        return str(round(psutil.virtual_memory().total / (1024 ** 3))) + 'GB'
    elif sys.platform == 'darwin':
        return unsupported_exception()
    else:
        return unsupported_exception()


def ram_manufacturer():
    if sys.platform == 'win32':
        manufacturer = subprocess.check_output('wmic memorychip get manufacturer').decode().split('\n')[1].strip()
        return manufacturer
    elif sys.platform == 'darwin':
        return unsupported_exception()
    elif sys.platform == 'linux':
        return sudo_dmidecode('Manufacturer').replace('	', '').replace('Manufacturer: ', '')
    else:
        return unsupported_exception()


def ram_serial_number():
    if sys.platform == 'win32':
        serialnumber = subprocess.check_output('wmic memorychip get serialnumber').decode().split('\n')[1].strip()
        return serialnumber
    elif sys.platform == 'darwin':
        return unsupported_exception()
    elif sys.platform == 'linux':
        return sudo_dmidecode('Serial Number').replace('	', '').replace('Serial Number: ', '')
    else:
        return unsupported_exception()


def ram_memory_type():
    if sys.platform == 'win32':
        memorytype = subprocess.check_output('wmic memorychip get memorytype').decode().split('\n')[1].strip()
        types = ['Unknown', 'Other', 'DRAM', 'Synchronous DRAM', 'Cache DRAM', 'EDO', 'EDRAM', 'VRAM',
                 'SRAM', 'RAM', 'ROM', 'Flash', 'EEPROM', 'FEPROM', 'EPROM', 'CDRAM', '3DRAM', 'SDRAM',
                 'SGRAM', 'RDRAM', 'DDR', 'DDR2', 'DDR2 FB-DIMM', 'DDR3', 'FBD2', 'DDR4']
        return str(types[int(memorytype)])
    elif sys.platform == 'darwin':
        return unsupported_exception()
    elif sys.platform == 'linux':
        return sudo_dmidecode('Type').replace('	', '').replace('Type: ', '')
    else:
        return unsupported_exception()


def ram_form_factor():
    if sys.platform == 'win32':
        factor = subprocess.check_output('wmic memorychip get memorytype').decode().split('\n')[1].strip()
        factors = ['Unknown', 'Other', 'SIP', 'DIP', 'ZIP', 'SOJ', 'Proprietary', 'SIMM',
                   'DIMM', 'TSOP', 'PGA', 'RIMM', 'SODIMM', 'SRIMM', 'SMD', 'SSMP', 'QFP',
                   'TQFP', 'SOIC', 'LCC', 'PLCC', 'BGA', 'FPBGA', 'LGA', 'FB-DIMM']
        return str(factors[int(factor)])
    elif sys.platform == 'darwin':
        return unsupported_exception()
    elif sys.platform == 'linux':
        return sudo_dmidecode('Form Factor').replace('	', '').replace('Form Factor: ', '')
    else:
        return unsupported_exception()


def ram_clockspeed():
    if sys.platform == 'win32':
        clockspeed = subprocess.check_output('wmic memorychip get speed').decode().split('\n')[1].strip()
        return str(clockspeed) + 'Hz'
    elif sys.platform == 'darwin':
        return unsupported_exception()
    elif sys.platform == 'linux':
        return sudo_dmidecode('Speed').replace('	', '').replace('Speed: ', '')
    else:
        return unsupported_exception()


def ram_usage():
    if sys.platform == 'win32' or 'linux':
        return str(psutil.virtual_memory()[2]) + '%'
    elif sys.platform == 'darwin':
        return unsupported_exception()
    else:
        return unsupported_exception()



from tkinter import *


class GFG:
    def __init__(self, master=None):
        self.canvas = None
        self.master = master
        self.create()

    def create(self):
        self.canvas = Canvas(self.master)

        def draw_gui():
            def draw_os_panel():
                def _main():
                    self.canvas.create_text(290, 15, fill="black", font=("Arial", 12),
                                            text="Operating System")
                    self.canvas.create_line(0, 30, 700, 30)

                def _data():
                    self.canvas.create_text(35, 40, fill="black", font=("Arial", 10),
                                            text="os name: ")
                    self.canvas.create_text(39, 55, fill="black", font=("Arial", 10),
                                            text="os version: ")
                    self.canvas.create_text(41, 70, fill="black", font=("Arial", 10),
                                            text="linux distro: ")
                    self.canvas.create_text(42, 85, fill="black", font=("Arial", 10),
                                            text="os platform: ")
                    self.canvas.create_text(40, 100, fill="black", font=("Arial", 10),
                                            text="os release: ")
                    self.canvas.create_text(53, 115, fill="black", font=("Arial", 10),
                                            text="os architecture: ")
                    self.canvas.create_text(38, 130, fill="black", font=("Arial", 10),
                                            text="os uptime: ")

                _main()
                _data()

            def draw_cpu_panel():
                def _main():
                    self.canvas.create_line(0, 140, 700, 140)
                    self.canvas.create_text(290, 155, fill="black", font=("Arial", 12),
                                            text="CPU")
                    self.canvas.create_line(0, 170, 700, 170)

                def _data():
                    self.canvas.create_text(38, 180, fill="black", font=("Arial", 10),
                                            text="cpu model: ")
                    self.canvas.create_text(53, 195, fill="black", font=("Arial", 10),
                                            text="cpu clockspeed: ")
                    self.canvas.create_text(54, 210, fill="black", font=("Arial", 10),
                                            text="cpu architecture: ")
                    self.canvas.create_text(72, 225, fill="black", font=("Arial", 10),
                                            text="cpu processor number: ")
                    self.canvas.create_text(38, 240, fill="black", font=("Arial", 10),
                                            text="cpu usage: ")
                    self.canvas.create_text(55, 255, fill="black", font=("Arial", 10),
                                            text="cpu temperature: ")
                    self.canvas.create_text(46, 270, fill="black", font=("Arial", 10),
                                            text="cpu vendor id: ")

                _main()
                _data()

            def draw_gpu_panel():
                def _main():
                    self.canvas.create_line(0, 280, 700, 280)
                    self.canvas.create_text(290, 295, fill="black", font=("Arial", 12),
                                            text="GPU")
                    self.canvas.create_line(0, 310, 700, 310)

                def _data():
                    self.canvas.create_text(38, 320, fill="black", font=("Arial", 10),
                                            text="gpu id: ")
                    self.canvas.create_text(53, 335, fill="black", font=("Arial", 10),
                                            text="gpu name: ")
                    self.canvas.create_text(54, 350, fill="black", font=("Arial", 10),
                                            text="gpu load: ")
                    self.canvas.create_text(72, 365, fill="black", font=("Arial", 10),
                                            text="gpu total memory: ")
                    self.canvas.create_text(38, 380, fill="black", font=("Arial", 10),
                                            text="gpu free memory: ")
                    self.canvas.create_text(55, 395, fill="black", font=("Arial", 10),
                                            text="gpu used memory: ")
                    self.canvas.create_text(46, 410, fill="black", font=("Arial", 10),
                                            text="gpu temperature: ")

                _main()
                _data()

            def draw_ram_panel():
                def _main():
                    self.canvas.create_line(0, 420, 700, 420)
                    self.canvas.create_text(290, 435, fill="black", font=("Arial", 12),
                                            text="RAM")
                    self.canvas.create_line(0, 450, 700, 450)

                def _data():
                    self.canvas.create_text(38, 460, fill="black", font=("Arial", 10),
                                            text="ram total memory: ")
                    self.canvas.create_text(53, 475, fill="black", font=("Arial", 10),
                                            text="ram manufacturer: ")
                    self.canvas.create_text(54, 490, fill="black", font=("Arial", 10),
                                            text="ram serial number: ")
                    self.canvas.create_text(72, 505, fill="black", font=("Arial", 10),
                                            text="ram memory type: ")
                    self.canvas.create_text(38, 520, fill="black", font=("Arial", 10),
                                            text="ram form factor: ")
                    self.canvas.create_text(55, 535, fill="black", font=("Arial", 10),
                                            text="ram clockspeed: ")
                    self.canvas.create_text(46, 550, fill="black", font=("Arial", 10),
                                            text="ram usage: ")

                _main()
                _data()

            def draw_storage_panel():
                def _main():
                    self.canvas.create_line(0, 560, 700, 560)
                    self.canvas.create_text(290, 575, fill="black", font=("Arial", 12),
                                            text="Storage")
                    self.canvas.create_line(0, 590, 700, 590)

                def _data():
                    self.canvas.create_text(38, 600, fill="black", font=("Arial", 10),
                                            text="drive_list: ")
                    self.canvas.create_text(53, 615, fill="black", font=("Arial", 10),
                                            text="total space: ")
                    self.canvas.create_text(54, 630, fill="black", font=("Arial", 10),
                                            text="used_space: ")
                    self.canvas.create_text(72, 645, fill="black", font=("Arial", 10),
                                            text="free space: ")
                    self.canvas.create_text(38, 660, fill="black", font=("Arial", 10),
                                            text="used space percent: ")
                    self.canvas.create_text(55, 675, fill="black", font=("Arial", 10),
                                            text="drive fstype: ")
                    self.canvas.create_text(46, 690, fill="black", font=("Arial", 10),
                                            text="drive mountpoint: ")

                _main()
                _data()

            draw_os_panel()
            draw_cpu_panel()
            draw_gpu_panel()
            draw_ram_panel()
            draw_storage_panel()

        draw_gui()
        self.canvas.pack(fill=BOTH, expand=True)


if __name__ == "__main__":
    root = Tk()
    geeks = GFG(root)
    root.title("pysil system information library gui extension")
    root.geometry("600x700")
    root.resizable(False, False)
    root.mainloop()



# vim: set et sw=4 sts=4 fileencoding=utf-8:
#
# The colorzero color library
#
# Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
#
# SPDX-License-Identifier: BSD-3-Clause

"""
Defines the classes for manipulating the attributes of the :class:`Color`
class through the standard binary operators.
"""

from math import pi


class Red(float):
    """
    Represents the red component of a :class:`Color` for use in
    transformations. Instances of this class can be constructed directly with a
    float value, or by querying the :attr:`Color.red` attribute. Addition,
    subtraction, and multiplication are supported with :class:`Color`
    instances. For example::

        >>> Color.from_rgb(0, 0, 0) + Red(0.5)
        <Color html='#800000' rgb=(0.5, 0, 0)>
        >>> Color('#f00') - Color('#900').red
        <Color html='#660000' rgb=(0.4, 0, 0)>
        >>> (Red(0.1) * Color('red')).red
        Red(0.1)
    """

    def __repr__(self):
        return "Red({:g})".format(self)


class Green(float):
    """
    Represents the green component of a :class:`Color` for use in
    transformations.  Instances of this class can be constructed directly with
    a float value, or by querying the :attr:`Color.green` attribute. Addition,
    subtraction, and multiplication are supported with :class:`Color`
    instances. For example::

        >>> Color(0, 0, 0) + Green(0.1)
        <Color html='#001a00' rgb=(0, 0.1, 0)>
        >>> Color.from_yuv(1, -0.4, -0.6) - Green(1)
        <Color html='#510030' rgb=(0.316098, 0, 0.187156)>
        >>> (Green(0.5) * Color('white')).rgb
        RGB(r=1.0, g=0.5, b=1.0)
    """

    def __repr__(self):
        return "Green({:g})".format(self)


class Blue(float):
    """
    Represents the blue component of a :class:`Color` for use in
    transformations.  Instances of this class can be constructed directly with
    a float value, or by querying the :attr:`Color.blue` attribute. Addition,
    subtraction, and multiplication are supported with :class:`Color`
    instances. For example::

        >>> Color(0, 0, 0) + Blue(0.2)
        <Color html='#000033' rgb=(0, 0, 0.2)>
        >>> Color.from_hls(0.5, 0.5, 1.0) - Blue(1)
        <Color html='#00ff00' rgb=(0, 1, 0)>
        >>> Blue(0.9) * Color('white')
        <Color html='#ffffe6' rgb=(1, 1, 0.9)>
    """

    def __repr__(self):
        return "Blue({:g})".format(self)


class Hue(float):
    """
    Represents the hue of a :class:`Color` for use in transformations.
    Instances of this class can be constructed directly with a float value in
    the range [0.0, 1.0) representing an angle around the `HSL hue wheel`_. As
    this is a circular mapping, 0.0 and 1.0 effectively mean the same thing,
    i.e.  out of range values will be normalized into the range [0.0, 1.0).

    The class can also be constructed with the keyword arguments ``deg`` or
    ``rad`` if you wish to specify the hue value in degrees or radians instead,
    respectively. Instances can also be constructed by querying the
    :attr:`Color.hue` attribute.

    Addition, subtraction, and multiplication are supported with :class:`Color`
    instances. For example::

        >>> Color(1, 0, 0).hls
        HLS(h=0.0, l=0.5, s=1.0)
        >>> (Color(1, 0, 0) + Hue(deg=180)).hls
        HLS(h=0.5, l=0.5, s=1.0)

    Note that whilst multiplication by a :class:`Hue` doesn't make much sense,
    it is still supported. However, the circular nature of a hue value can lead
    to suprising effects. In particular, since 1.0 is equivalent to 0.0 the
    following may be observed::

        >>> (Hue(1.0) * Color.from_hls(0.5, 0.5, 1.0)).hls
        HLS(h=0.0, l=0.5, s=1.0)

    .. _HSL hue wheel: https://en.wikipedia.org/wiki/Hue
    """

    def __new__(cls, n=None, deg=None, rad=None):
        if n is not None:
            return super().__new__(cls, n % 1.0)
        elif deg is not None:
            return super().__new__(cls, (deg / 360.0) % 1.0)
        elif rad is not None:
            return super().__new__(cls, (rad / (2 * pi)) % 1.0)
        else:
            raise ValueError('You must specify a value, or deg or rad')

    def __repr__(self):
        return "Hue(deg={self.deg:g})".format(self=self)

    @property
    def deg(self):
        """
        Returns the hue as a value in degrees with the range 0.0 <= n < 360.0.
        """
        return self * 360.0

    @property
    def rad(self):
        """
        Returns the hue as a value in radians with the range 0.0 <= n < 2π.
        """
        return self * 2 * pi


class Lightness(float):
    """
    Represents the lightness of a :class:`Color` for use in transformations.
    Instances of this class can be constructed directly with a float value, or
    by querying the :attr:`Color.lightness` attribute. Addition, subtraction,
    and multiplication are supported with :class:`Color` instances. For
    example::

        >>> Color(0, 0, 0) + Lightness(0.1)
        <Color html='#1a1a1a' rgb=(0.1, 0.1, 0.1)>
        >>> Color.from_rgb_bytes(0x80, 0x80, 0) - Lightness(0.2)
        <Color html='#1a1a00' rgb=(0.101961, 0.101961, 0)>
        >>> Lightness(0.9) * Color('wheat')
        <Color html='#f0ce8e' rgb=(0.94145, 0.806785, 0.555021)>
    """

    def __repr__(self):
        return "Lightness({:g})".format(self)


class Saturation(float):
    """
    Represents the saturation of a :class:`Color` for use in transformations.
    Instances of this class can be constructed directly with a float value, or
    by querying the :attr:`Color.saturation` attribute. Addition, subtraction,
    and multiplication are supported with :class:`Color` instances. For
    example::

        >>> Color(0.9, 0.9, 0.6) + Saturation(0.1)
        <Color html='#ecec93' rgb=(0.925, 0.925, 0.575)>
        >>> Color('red') - Saturation(1)
        <Color html='#808080' rgb=(0.5, 0.5, 0.5)>
        >>> Saturation(0.5) * Color('wheat')
        <Color html='#e4d9c3' rgb=(0.896078, 0.85098, 0.766667)>
    """

    def __repr__(self):
        return "Saturation({:g})".format(self)


class Luma(float):
    """
    Represents the luma of a :class:`Color` for use in transformations.
    Instances of this class can be constructed directly with a float value, or
    by querying the :attr:`Color.yuv.y` attribute. Addition, subtraction, and
    multiplication are supported with :class:`Color` instances. For example::

        >>> Color(0, 0, 0) + Luma(0.1)
        <Color html='#1a1a1a' rgb=(0.1, 0.1, 0.1)>
        >>> Color('red') * Luma(0.5)
        <Color html='#d90000' rgb=(0.8505, 0, 0)>
    """

    def __repr__(self):
        return "Luma({:g})".format(self)



import platform
import re
import sys
from core.exception import *


def machine_name():
    if sys.platform == 'win32' or 'linux':
        return platform.node()
    elif sys.platform == 'darwin':
        return unsupported_exception()
    else:
        return unsupported_exception()


def bios_type():
    if sys.platform == 'win32':
        with open(r'C:\Windows\Panther\setupact.log') as f:
            pattern = re.compile(r'Detected boot environment: (\w+)')
            for line in f:
                match = pattern.search(line)
                if match:
                    boot_type = match.group(1).upper()
                    if boot_type == 'EFI':
                        return 'UEFI'
                    else:
                        return 'BIOS'
    elif sys.platform == 'darwin':
        return unsupported_exception()
    elif sys.platform == 'linux':
        try:
            open("/sys/firmware/efi")
            return 'UEFI'
        except IOError:
            return 'BIOS'
    else:
        return unsupported_exception()



import os
import sys
import win32com.client
from core.exception import *


def get_usb_list():
    if sys.platform == 'win32':
        wmi = win32com.client.GetObject("winmgmts:")
        for usb in wmi.InstancesOf("Win32_USBHub"):
            return usb.DeviceID
    elif sys.platform == 'darwin':
        return
    elif sys.platform == 'linux':
        return os.popen('lsusb').read()
    else:
        return unsupported_exception()



# vim: set et sw=4 sts=4 fileencoding=utf-8:
#
# The colorzero color library
#
# Copyright (c) 2018 Dave Jones <dave@waveform.org.uk>
#
# SPDX-License-Identifier: BSD-3-Clause

"Defines the various algorithms for :meth:`Color.difference`."

from math import sqrt, atan2, degrees, radians, sin, cos, exp

# Lots of the delta-e functions use single character parameter names and
# variables internally; this is is normal and in keeping with most of the
# referenced sources
# pylint: disable=invalid-name


def euclid(color1, color2):
    """
    Calculates color difference as a simple `Euclidean distance`_ by treating
    the three components as spatial dimensions.

    .. note::

        This function will return considerably different values to the other
        difference functions. In particular, the maximum "difference" will be
        :math:`\\sqrt{3}` which is much smaller than the output of the CIE
        functions.

    .. _Euclidean distance: https://en.wikipedia.org/wiki/Euclidean_distance
    """
    return sqrt(sum((e1 - e2) ** 2 for e1, e2 in zip(color1, color2)))


def cie1976(color1, color2):
    """
    Calculates color difference according to the `CIE 1976`_ formula.
    Effectively this is the Euclidean formula, but with CIE L*a*b* components
    instead of RGB.

    .. _CIE 1976: https://en.wikipedia.org/wiki/Color_difference#CIE76
    """
    return sqrt(sum((e1 - e2) ** 2 for e1, e2 in zip(color1, color2)))


def cie1994(color1, color2, method):
    """
    Calculates color difference according to the `CIE 1994`_ formula. The
    *method* can be either "cie1994g" for the "graphical" biases, or "cie1994t"
    for the "textile" biases. The CIE1994 is also basically the Euclidean
    formula (with biases) but in CIE L*C*H* space.

    .. _CIE 1994: https://en.wikipedia.org/wiki/Color_difference#CIE94
    """
    C1 = sqrt(color1.a ** 2 + color1.b ** 2)
    C2 = sqrt(color2.a ** 2 + color2.b ** 2)

    dL = color1.l - color2.l
    dC = C1 - C2
    # Don't bother with the sqrt here as due to limited float precision
    # we can wind up with a domain error (because the value is ever so
    # slightly negative - try it with black'n'white for an example), and we're
    # just going to square the result in the final equation anyway
    dH2 = (color1.a - color2.a) ** 2 + (color1.b - color2.b) ** 2 - dC ** 2

    kL, K1, K2 = {
        'cie1994g': (1, 0.045, 0.015),
        'cie1994t': (2, 0.048, 0.014),
    }[method]
    SC = 1 + K1 * C1
    SH = 1 + K2 * C1
    return sqrt(
        (dL / kL) ** 2 +
        (dC / SC) ** 2 +
        (dH2 / SH ** 2)
    )


def cie1994g(color1, color2):
    """
    Calculates color difference according to the `CIE 1994`_ formula with the
    "textile" bias. See :func:`cie1994` for further information.

    .. _CIE 1994: https://en.wikipedia.org/wiki/Color_difference#CIE94
    """
    return cie1994(color1, color2, 'cie1994g')


def cie1994t(color1, color2):
    """
    Calculates color difference according to the `CIE 1994`_ formula with the
    "graphics" bias. See :func:`cie1994` for further information.

    .. _CIE 1994: https://en.wikipedia.org/wiki/Color_difference#CIE94
    """
    return cie1994(color1, color2, 'cie1994t')


def ciede2000(color1, color2):
    """
    Calculates color difference according to the `CIEDE 2000`_ formula. This is
    the most accurate algorithm currently implemented but also the most complex
    and slowest. Like CIE1994 it is largely based in CIE L*C*h* space, but with
    several modifications to account for perceptual uniformity flaws.

    .. _CIEDE 2000: https://en.wikipedia.org/wiki/Color_difference#CIEDE2000
    """
    # See WP article and Sharma 2005 for important implementation notes:
    # http://www.ece.rochester.edu/~gsharma/ciede2000/ciede2000noteCRNA.pdf
    #
    # Yes, there's lots of locals; but this is easiest to understand as it's a
    # near straight translation of the math
    # pylint: disable=too-many-locals

    C_ = (
        sqrt(color1.a ** 2 + color1.b ** 2) +
        sqrt(color2.a ** 2 + color2.b ** 2)
    ) / 2

    G = (1 - sqrt(C_ ** 7 / (C_ ** 7 + 25 ** 7))) / 2
    a1_prime = (1 + G) * color1.a
    a2_prime = (1 + G) * color2.a

    C1_prime = sqrt(a1_prime ** 2 + color1.b ** 2)
    C2_prime = sqrt(a2_prime ** 2 + color2.b ** 2)
    L_ = (color1.l + color2.l) / 2
    C_ = (C1_prime + C2_prime) / 2

    h1 = (
        0.0 if color1.b == a1_prime == 0 else
        degrees(atan2(color1.b, a1_prime)) % 360
    )
    h2 = (
        0.0 if color2.b == a2_prime == 0 else
        degrees(atan2(color2.b, a2_prime)) % 360
    )
    if C1_prime * C2_prime == 0.0:
        dh = 0.0
        h_ = h1 + h2
    elif abs(h1 - h2) <= 180:
        dh = h2 - h1
        h_ = (h1 + h2) / 2
    else:
        if h2 > h1:
            dh = h2 - h1 - 360
        else:
            dh = h2 - h1 + 360
        if h1 + h2 >= 360:
            h_ = (h1 + h2 - 360) / 2
        else:
            h_ = (h1 + h2 + 360) / 2

    dL = color2.l - color1.l
    dC = C2_prime - C1_prime
    dH = 2 * sqrt(C1_prime * C2_prime) * sin(radians(dh / 2))

    T = (
        1 -
        0.17 * cos(radians(h_ - 30)) +
        0.24 * cos(radians(2 * h_)) +
        0.32 * cos(radians(3 * h_ + 6)) -
        0.20 * cos(radians(4 * h_ - 63))
    )
    SL = 1 + (0.015 * (L_ - 50) ** 2) / sqrt(20 + (L_ - 50) ** 2)
    SC = 1 + 0.045 * C_
    SH = 1 + 0.015 * C_ * T
    RT = (
        -2 * sqrt(C_ ** 7 / (C_ ** 7 + 25 ** 7)) *
        sin(radians(60 * exp(-(((h_ - 275) / 25) ** 2))))
    )

    return sqrt(
        (dL / SL) ** 2 +
        (dC / SC) ** 2 +
        (dH / SH) ** 2 +
        RT * (dC / SC) * (dH / SH)
    )



'''PyEDID module'''

import struct
from typing import Optional, Union

from .types import Edid, Registry, DEFAULT_REGISTRY
from .types.edid import _RAW_EDID, _EDID_STRUCT_FORMAT, _ASPECT_RATIOS, _TIMINGS
from .helpers import get_edid_from_xrandr_verbose

__all__ = (
    'Edid',
    'Registry',
    'DEFAULT_REGISTRY',
    'get_edid_from_xrandr_verbose',
    'parse_edid'
)

__version__ = '1.0.1'


def parse_edid(raw: Union[bytes, str], registry: Registry = DEFAULT_REGISTRY) -> Edid:
    '''Parse raw EDID and return parsed object

    :param raw: Raw edid value
    :param registry: Edid registry, default uses the DEFAULT_REGISTRY

    :raises ValueError: bad Edid format
    :raises TypeError: bad inputs
    '''
    if not isinstance(raw, bytes):
        if isinstance(raw, str):
            raw = bytes.fromhex(raw)
        else:
            raise TypeError('Bad EDID type')

    if not isinstance(registry, Registry):
        raise TypeError('Bad registry type')

    # crop additional edid
    if len(raw) > 128:
        raw = raw[:128]

    if sum(raw) % 256 != 0:
        raise ValueError('Checksum mismatch')

    if raw[:8] != b'\x00\xff\xff\xff\xff\xff\xff\x00':
        raise ValueError('Invalid EDID header')

    unpacked = struct.unpack(_EDID_STRUCT_FORMAT, raw)
    raw_edid = _RAW_EDID(*unpacked)

    resolutions = []
    for i in range(16):
        bit = raw_edid.timings_supported & i
        if bit:
            resolutions.append(_TIMINGS[16-i])

    for i in range(8):
        bytes_data = raw_edid.timings_edid[2*i:2*i+2]
        if bytes_data == b'\x01\x01':
            continue
        byte1, byte2 = bytes_data
        x_res = 8*(int(byte1)+31)
        aspect_ratio = _ASPECT_RATIOS[(byte2>>6) & 0b11]
        y_res = int(x_res * aspect_ratio[1]/aspect_ratio[0])
        rate = (int(byte2) & 0b00111111) + 60.0
        resolutions.append((x_res, y_res, rate))

    name: Optional[str] = None
    serial: Optional[str] = None

    for timing_bytes in (raw_edid.timing_1, raw_edid.timing_2, raw_edid.timing_3, raw_edid.timing_4):
        # 'other' descriptor
        if timing_bytes[0:2] == b'\x00\x00':
            timing_type = timing_bytes[3]
            if timing_type in (0xFF, 0xFE, 0xFC):
                buffer = timing_bytes[5:]
                buffer = buffer.partition(b'\x0a')[0]
                text = buffer.decode('cp437')
                if timing_type == 0xFF:
                    serial = text
                elif timing_type == 0xFC:
                    name = text

    return Edid(
        manufacturer_id = raw_edid.manu_id,
        manufacturer = registry.get_company_by_raw(raw_edid.manu_id),
        product_id = raw_edid.prod_id,
        year = raw_edid.manu_year + 1990,
        week = raw_edid.manu_week,
        edid_version = '{:d}.{:d}'.format(raw_edid.edid_version, raw_edid.edid_revision),
        type = 'digital' if (raw_edid.input_type & 0xFF) else 'analog',
        width = float(raw_edid.width),
        height = float(raw_edid.height),
        gamma = (raw_edid.gamma + 100) / 100,
        dpms_standby = bool(raw_edid.features & 0xFF),
        dpms_suspend = bool(raw_edid.features & 0x7F),
        dpms_activeoff = bool(raw_edid.features & 0x3F),
        resolutions = resolutions,
        name = name,
        serial = serial if serial else raw_edid.serial_no
    )



# vim: set et sw=4 sts=4 fileencoding=utf-8:
#
# The colorzero color library
#
# Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
#
# SPDX-License-Identifier: BSD-3-Clause

"Define the tuples used to represent various color systems."

from collections import namedtuple, OrderedDict

from .attr import Red, Green, Blue, Hue, Lightness, Saturation, Luma


class RGB(tuple):
    "Named tuple representing red, green, and blue."
    # NOTE: This source mostly generated by namedtuple(..., verbose=True)
    # XXX When py3.6+ required, replace with derivative of typing.NamedTuple

    __slots__ = ()

    _fields = ('r', 'g', 'b')

    def __new__(cls, r, g, b):
        return tuple.__new__(cls, (r, g, b))

    def _replace(self, **kw):
        "Return a new RGB object replacing specified fields with new values"
        result = tuple.__new__(RGB, map(kw.pop, 'rgb', self))
        if kw:
            raise ValueError(
                'Got unexpected field names: {!r}'.format(list(kw)))
        return result

    def __repr__(self):
        "Return a nicely formatted representation string"
        return (
            self.__class__.__name__ +
            '(r={self.r:g}, g={self.g:g}, b={self.b:g})'.format(self=self))

    def _asdict(self):
        "Return a new OrderedDict which maps field names to their values."
        return OrderedDict(zip(self._fields, self))

    def __getnewargs__(self):
        "Return self as a plain tuple.  Used by copy and pickle."
        return tuple(self)

    @property
    def r(self):
        "Return the red value as a :class:`Red` instance"
        # pylint: disable=invalid-name
        return Red(self[0])

    @property
    def red(self):
        "Return the red value as a :class:`Red` instance"
        return Red(self[0])

    @property
    def g(self):
        "Return the green value as a :class:`Green` instance"
        # pylint: disable=invalid-name
        return Green(self[1])

    @property
    def green(self):
        "Return the green value as a :class:`Green` instance"
        return Green(self[1])

    @property
    def b(self):
        "Return the blue value as a :class:`Blue` instance"
        # pylint: disable=invalid-name
        return Blue(self[2])

    @property
    def blue(self):
        "Return the blue value as a :class:`Blue` instance"
        return Blue(self[2])


class HLS(tuple):
    "Named tuple representing hue, lightness, and saturation."
    # NOTE: This source mostly generated by namedtuple(..., verbose=True)
    # XXX When py3.6+ required, replace with derivative of typing.NamedTuple

    __slots__ = ()

    _fields = ('h', 'l', 's')

    def __new__(cls, h, l, s):
        return tuple.__new__(cls, (h, l, s))

    def _replace(self, **kw):
        "Return a new HLS object replacing specified fields with new values"
        result = tuple.__new__(HLS, map(kw.pop, 'hls', self))
        if kw:
            raise ValueError('Got unexpected field names: {!r}'.format(list(kw)))
        return result

    def __repr__(self):
        "Return a nicely formatted representation string"
        return (
            self.__class__.__name__ +
            '(h={self.h:g}, l={self.l:g}, s={self.s:g})'.format(self=self))

    def _asdict(self):
        "Return a new OrderedDict which maps field names to their values."
        return OrderedDict(zip(self._fields, self))

    def __getnewargs__(self):
        "Return self as a plain tuple.  Used by copy and pickle."
        return tuple(self)

    @property
    def h(self):
        "Return the hue value as a :class:`Hue` instance"
        # pylint: disable=invalid-name
        return Hue(self[0])

    @property
    def hue(self):
        "Return the hue value as a :class:`Hue` instance"
        return Hue(self[0])

    @property
    def l(self):
        "Return the lightness value as a :class:`Lightness` instance"
        # pylint: disable=invalid-name
        return Lightness(self[1])

    @property
    def lightness(self):
        "Return the lightness value as a :class:`Lightness` instance"
        return Lightness(self[1])

    @property
    def s(self):
        "Return the saturation value as a :class:`Saturation` instance"
        # pylint: disable=invalid-name
        return Saturation(self[2])

    @property
    def saturation(self):
        "Return the saturation value as a :class:`Saturation` instance"
        return Saturation(self[2])


class HSV(tuple):
    'Named tuple representing hue, saturation, and value ("brightness").'
    # XXX When py3.6+ required, replace with derivative of typing.NamedTuple

    __slots__ = ()

    _fields = ('h', 's', 'v')

    def __new__(cls, h, s, v):
        return tuple.__new__(cls, (h, s, v))

    def _replace(self, **kw):
        "Return a new HSV object replacing specified fields with new values"
        result = tuple.__new__(HSV, map(kw.pop, 'hsv', self))
        if kw:
            raise ValueError(
                'Got unexpected field names: {!r}'.format(list(kw)))
        return result

    def __repr__(self):
        "Return a nicely formatted representation string"
        return (
            self.__class__.__name__ +
            '(h={self.h:g}, s={self.s:g}, v={self.v:g})'.format(self=self))

    def _asdict(self):
        "Return a new OrderedDict which maps field names to their values."
        return OrderedDict(zip(self._fields, self))

    def __getnewargs__(self):
        "Return self as a plain tuple.  Used by copy and pickle."
        return tuple(self)

    @property
    def h(self):
        "Return the hue value as a :class:`Hue` instance"
        # pylint: disable=invalid-name
        return Hue(self[0])

    @property
    def hue(self):
        "Return the hue value as a :class:`Hue` instance"
        return Hue(self[0])

    @property
    def s(self):
        "Return the saturation value as a :class:`Saturation` instance"
        # pylint: disable=invalid-name
        return Saturation(self[1])

    @property
    def saturation(self):
        "Return the saturation value as a :class:`Saturation` instance"
        return Saturation(self[1])

    @property
    def v(self):
        "Return the brightness value"
        return self[2]

    @property
    def value(self):
        "Return the brightness value"
        return self[2]


class YUV(tuple):
    "Named tuple representing luma and two chroma offsets"
    # XXX When py3.6+ required, replace with derivative of typing.NamedTuple

    __slots__ = ()

    _fields = ('y', 'u', 'v')

    def __new__(cls, y, u, v):
        return tuple.__new__(cls, (y, u, v))

    def _replace(self, **kw):
        "Return a new YUV object replacing specified fields with new values"
        result = tuple.__new__(YUV, map(kw.pop, 'yuv', self))
        if kw:
            raise ValueError(
                'Got unexpected field names: {!r}'.format(list(kw)))
        return result

    def __repr__(self):
        "Return a nicely formatted representation string"
        return (
            self.__class__.__name__ +
            '(y={self.y:g}, u={self.u:g}, v={self.v:g})'.format(self=self))

    def _asdict(self):
        "Return a new OrderedDict which maps field names to their values."
        return OrderedDict(zip(self._fields, self))

    def __getnewargs__(self):
        "Return self as a plain tuple.  Used by copy and pickle."
        return tuple(self)

    @property
    def y(self):
        "Return the luma value as a :class:`Luma` instance"
        # pylint: disable=invalid-name
        return Luma(self[0])

    @property
    def luma(self):
        "Return the luma value as a :class:`Luma` instance"
        return Luma(self[0])

    @property
    def u(self):
        "Return the first chroma offset"
        # pylint: disable=invalid-name
        return self[1]

    @property
    def v(self):
        "Return the second chroma offset"
        # pylint: disable=invalid-name
        return self[2]


class CMY(namedtuple('CMY', ('c', 'm', 'y'))):
    "Named tuple representing cyan, magenta, and yellow."
    # XXX When py3.6+ required, replace with derivative of typing.NamedTuple

    @property
    def cyan(self):
        # pylint: disable=missing-docstring
        return self.c

    @property
    def magenta(self):
        # pylint: disable=missing-docstring
        return self.m

    @property
    def yellow(self):
        # pylint: disable=missing-docstring
        return self.y


class CMYK(namedtuple('CMYK', ('c', 'm', 'y', 'k'))):
    "Named tuple representing cyan, magenta, yellow, and black."
    # XXX When py3.6+ required, replace with derivative of typing.NamedTuple

    @property
    def cyan(self):
        # pylint: disable=missing-docstring
        return self.c

    @property
    def magenta(self):
        # pylint: disable=missing-docstring
        return self.m

    @property
    def yellow(self):
        # pylint: disable=missing-docstring
        return self.y

    @property
    def black(self):
        # pylint: disable=missing-docstring
        return self.k


YIQ = namedtuple('YIQ', ('y', 'i', 'q'))
XYZ = namedtuple('XYZ', ('x', 'y', 'z'))
Luv = namedtuple('Luv', ('l', 'u', 'v'))
Lab = namedtuple('Lab', ('l', 'a', 'b'))



'''PNP ID registry module'''

import csv
import string
from collections import UserDict
from html.parser import HTMLParser
from urllib.parse import urljoin
from typing import Optional

import requests
from requests.exceptions import HTTPError, ConnectionError, Timeout


__all__ = ('Registry')


class _WebPnpIdParser(HTMLParser):
    '''Parser for HTML table with pnp id data'''
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._find_table = False
        self._find_row = False
        # first -- company name, second -- pnp id, third -- approved date
        self._last_field = []
        # key -- pnp id, value -- tuple (company_name, approved_date)
        self.result = {}

    def handle_starttag(self, tag, attrs):
        if tag == 'tbody':
            self._find_table = True
        elif self._find_table and tag == 'tr':
            self._find_row = True

    def handle_endtag(self, tag):
        if tag == 'tbody':
            self._find_table = False
        elif self._find_table and tag == 'tr':
            self._find_row = False
            # add table row to result
            self.result[self._last_field[1]] = (self._last_field[0], self._last_field[-1])
            self._last_field.clear()

    def handle_data(self, data):
        # skip processing until table was found
        if not self._find_table:
            return

        if self._find_row:
            data = data.strip()
            if data:
                self._last_field.append(data)

    def error(self, message):
        super().close()


class Registry(UserDict):
    '''Registry PNP ID data

    key is pnp_id
    value is company name
    '''
    __PNP_BASE_URL = 'https://uefi.org'
    __PNP_EXPORT = '/uefi-pnp-export'
    __PNP_FILTER = '/pnp_id_list?search='
    __DEFAULT_NAME = 'Unknown'

    def __repr__(self) -> str:
        return f'Registry({len(self)} items)'

    @classmethod
    def from_web(cls, filter_by_id: Optional[str] = None) -> 'Registry':
        '''Return the registry updated from uefi.org

        If uefi.org return not 200 code then will returned empty registry

        :param filter_by_id: filtering by ID, (optional)

        :raises TypeError: bad argument type
        :raises RuntimeError: internet connection troubles
        '''
        if filter_by_id:
            if not isinstance(filter_by_id, str):
                raise TypeError

            url = urljoin(cls.__PNP_BASE_URL, cls.__PNP_FILTER)
            url += filter_by_id
        else:
            url = urljoin(cls.__PNP_BASE_URL, cls.__PNP_EXPORT)

        registry = cls()
        try:
            resp = requests.get(url)
            resp.raise_for_status()
        except HTTPError:
            return registry
        except (ConnectionError, Timeout) as err:
            raise RuntimeError from err

        parser = _WebPnpIdParser()
        parser.feed(resp.text)

        for key, value in parser.result.items():
            # skip invalid search value
            if filter_by_id and key != filter_by_id:
                continue
            registry[key] = value[0]
        return registry

    @classmethod
    def from_csv(cls, path: str, filter_by_id: str = None) -> 'Registry':
        '''Load the registry from a local file in CSV format

        :param path: path to file
        :param filter_by_id: filter registry by id (optional)

        :raises TypeError: bad argument type
        '''
        if not isinstance(path, str):
            raise TypeError
        if filter_by_id:
            if not isinstance(filter_by_id, str):
                raise TypeError

        registry = cls()
        with open(path, 'r') as f:
            reader = csv.reader(f)
            for line in reader:
                # filter
                if filter_by_id and filter_by_id != line[0]:
                    continue
                registry[line[0]] = line[1]
        return registry

    def to_csv(self, path: str) -> None:
        '''Dump the registry to CSV file

        :raises TypeError: bad argument type
        '''
        if not isinstance(path, str):
            raise TypeError

        with open(path, 'w') as f:
            writer = csv.writer(f)
            writer.writerows(self.items())

    def get_company_by_pnp(self, pnp_id: str) -> str:
        '''Convert PNP id to company name or 'Unknown' if not found'''
        return self.get(pnp_id, self.__DEFAULT_NAME)

    def get_company_by_raw(self, raw_id: int) -> str:
        '''Convert raw edid value to company name or 'Unknown' if not found'''
        tmp = [(raw_id >> 10) & 31, (raw_id >> 5) & 31, raw_id & 31]
        try:
            pnp_id = ''.join(string.ascii_uppercase[n-1] for n in tmp)
            return self.get_company_by_pnp(pnp_id)
        except IndexError:
            return self.__DEFAULT_NAME
